package cis690.insight.debug;

import org.eclipse.debug.core.DebugException;
import org.eclipse.debug.core.DebugPlugin;
import org.eclipse.debug.core.ILaunchManager;
import org.eclipse.debug.core.model.IDebugTarget;
import org.eclipse.debug.core.model.IVariable;
import org.eclipse.jdt.debug.core.IJavaObject;
import org.eclipse.jdt.debug.core.IJavaStackFrame;
import org.eclipse.jdt.debug.core.IJavaThread;
import org.eclipse.jdt.internal.debug.core.model.JDIArrayEntryVariable;
import org.eclipse.jdt.internal.debug.core.model.JDIObjectValue;
import org.eclipse.jdt.internal.debug.core.model.JDIThread;
import org.eclipse.jdt.internal.debug.core.model.JDIValue;
import org.eclipse.jdt.internal.debug.core.model.JDIVariable;

/**
 * 
 * @version Apr 22, 2004
 * @author charlie
 */
public class DebugUtil
{
	public static IDebugTarget getDebugTarget()
	{
		DebugPlugin plug = DebugPlugin.getDefault();
		ILaunchManager lMan = plug.getLaunchManager();
		IDebugTarget[] targets = lMan.getDebugTargets();
		
		if (targets.length > 0)
		{
			return targets[0]; // assume there is only one target
		}
		else
		{
			return null;
		}
	}
	
	public static String getCurrentClassName(IJavaThread t)
	{
		String name;
		try
		{
			name = ((IJavaStackFrame)t.getTopStackFrame()).getDeclaringTypeName();
		}
		catch (DebugException e)
		{
			e.printStackTrace();
			name = "<<unnamed>>";	
		}
		
		return name;
	}
	
	public static String getCurrentMethodName(IJavaThread t)
	{
		String name;
		try
		{
			name = ((IJavaStackFrame)t.getTopStackFrame()).getMethodName();
		}
		catch (DebugException e)
		{
			e.printStackTrace();
			name = "<<unnamed>>";	
		}

		return name;
	}
	
	public static IVariable[] getVisibleVars(IJavaThread t)
	{
		IVariable[] vars;
		try
		{
			IVariable[] localVars;
			IVariable[] objVars;
			IJavaStackFrame sf = (IJavaStackFrame)t.getTopStackFrame();
			if (sf == null)
			{
				return new IVariable[0];
			}
			else
			{
			    localVars = sf.getLocalVariables();
			}
			
			IJavaObject obj = ((IJavaStackFrame)t.getTopStackFrame()).getThis();
			if (obj == null)
			{
			    objVars = new IVariable[0];
			}
			else
			{
			    objVars = obj.getVariables();
			}
			vars = new IVariable[localVars.length+objVars.length];
			System.arraycopy(objVars, 0, vars, 0, objVars.length);
			System.arraycopy(localVars, 0, vars, objVars.length, localVars.length);
		}
		catch (DebugException e)
		{
			e.printStackTrace();
			vars = new IVariable[0];
		}

		return vars;
	}
	
	public static IVariable[] getVarChildren(IVariable var)
	{
		try
		{
			return var.getValue().getVariables();
		}
		catch (DebugException e)
		{
			e.printStackTrace();
			return new IVariable[0];
		}
	}
	
	public static String getThreadName(IJavaThread t)
	{
		try
		{
			return t.getName();
		}
		catch (DebugException e)
		{
			e.printStackTrace();
			return "<<unknown name>>";
		}
	}
	
	public static String getVarName(IVariable v)
	{
		try
		{
			return v.getName();
		}
		catch (DebugException e)
		{
			e.printStackTrace();
			return "<<unknown var>>";
		}
	}
	
	public static String getVarTypeName(IVariable v)
	{
		try
		{
			return v.getReferenceTypeName();
		}
		catch (DebugException e)
		{
			e.printStackTrace();
			return "<<unknown type>>";
		}
	}
	
	public static String getVarValueStr(IVariable v)
	{
		try
		{
			return v.getValue().getValueString();
		}
		catch (DebugException e)
		{
			e.printStackTrace();
			return "<<unknown value>>";
		}
	}
	
	public static boolean isStr(IVariable v)
	{
		try
		{
			return v.getReferenceTypeName().equals("java.lang.String");
		}
		catch (DebugException e)
		{
			e.printStackTrace();
			return false;
		}
	}
	
	public static long getObjectID(Object obj)
	{	
		if (obj instanceof DebugObject)
		{
			return getObjectID(((DebugObject)obj).getObj());
		}
	
		if (obj instanceof JDIArrayEntryVariable)
		{
			try
			{
				return getObjectID(((JDIArrayEntryVariable)obj).getValue());
			}
			catch (DebugException e) {e.printStackTrace(); }
		}
		else if (obj instanceof JDIThread)
		{
			return ((JDIThread)obj).getUnderlyingThread().uniqueID();
		}
		else if (obj instanceof JDIVariable)
		{
			try
			{
				JDIValue val = (JDIValue)((JDIVariable)obj).getValue();
				if (val instanceof JDIObjectValue)
				{
					return ((JDIObjectValue)val).getUnderlyingObject().uniqueID();
				}
				else
				{
					// hack, chop, smash, embarassing...
					String str = val.getValueString();
					str = str.replaceAll("\\D", "");
					return Long.parseLong(str);
				}
			}
			catch (DebugException e)
			{
				e.printStackTrace();
			}
		}
		else if (obj instanceof JDIObjectValue)
		{
			return ((JDIObjectValue)obj).getUnderlyingObject().uniqueID();
		}
		
		System.err.println("Unable to ID Object: "+obj.getClass().getName());
		return obj.hashCode();	
	}
	
	public static boolean symanticEquality(DebugObject obj1, Object obj2)
	{
		final long obj1ID = getObjectID(obj1);
		final long obj2ID = getObjectID(obj2);
		return obj1ID == obj2ID;
		
	}
}
