package cis690.insight.parts;


import java.util.*;

import org.eclipse.draw2d.FreeformLayer;
import org.eclipse.draw2d.FreeformLayout;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.editparts.AbstractGraphicalEditPart;

import cis690.insight.debug.DebugModel;
import cis690.insight.debug.DebugObject;
import cis690.insight.debug.IDebugModel;
import cis690.insight.debug.IDebugObject;
import cis690.insight.figures.ObjConnection;
import cis690.insight.*;

/**
 * 
 * @version Apr 29, 2004
 * @author charlie
 */
public class DebugModelEditPart extends AbstractGraphicalEditPart
	implements Observer
{	
	private AsyncRefresh m_refresher;
	private Vector m_connections;
	
	public DebugModelEditPart(DebugModel model)
	{
		m_connections = new Vector();
		setModel(model);
		m_refresher = new AsyncRefresh(this);
	}
	
	protected IFigure createFigure()
	{
		IFigure layer = new FreeformLayer();
		layer.setLayoutManager(new FreeformLayout());
		layer.setBackgroundColor(Colors.BACKGROUND);
		layer.setOpaque(true);
		return layer;
	}
	
	public void activate()
	{
		super.activate();
		((Observable)getModel()).addObserver(this);	
	}
	
	public void deactivate()
	{
		super.deactivate();
		((Observable)getModel()).deleteObserver(this);	
	}

	protected void addChildVisual(EditPart child, int index)
	{
		super.addChildVisual(child, index);
		
		// give it some constraints
		DebugObjectEditPart part = (DebugObjectEditPart)child;
		DebugObject obj = (DebugObject)part.getModel();
		IFigure figure = part.getFigure();
		Point pt = obj.getXY();
		setLayoutConstraint(part, figure, new Rectangle(pt.x, pt.y, -1, -1));
	}
	
	public void refresh()
	{
		super.refresh();
		refreshConnections();
	}
	
	private void refreshConnections()
	{
		Vector curConnxs = new Vector();
		List childParts = getChildren();
		for (int i=0; i<childParts.size(); ++i)
		{
			DebugObjectEditPart objPart1 = (DebugObjectEditPart)childParts.get(i);
			List modelChildren = objPart1.getObjChildren();
			for (int j=0; j<modelChildren.size(); ++j)
			{
				IDebugObject obj = (IDebugObject)modelChildren.get(j);
				DebugObjectEditPart targetPart = getPartForModel(obj);
				if (targetPart == null)
				{
					continue;	
				}
				
				ObjConnection connx = new ObjConnection(objPart1.getModel(), obj);
				connx.setSourceAnchor(objPart1.getSourceAnchor(obj));
				connx.setTargetAnchor(targetPart.getTargetAnchor());
				
				if (curConnxs.contains(connx))
				{
					((ObjConnection)curConnxs.get(curConnxs.indexOf(connx))).renewTarget(connx);
				}
				else
				{
					curConnxs.add(connx);
				}
			}
		}
		
		for (int i=0; i<m_connections.size(); ++i)
		{
			getFigure().remove((IFigure)m_connections.get(i));	
		}
		
		m_connections.removeAllElements();
		m_connections.addAll(curConnxs);

		for (int i=0; i<m_connections.size(); ++i)
		{
			getFigure().add((IFigure)m_connections.get(i));
		}
	}
	
	private DebugObjectEditPart getPartForModel(IDebugObject model)
	{
		List children = getChildren();
		for (int i=0; i<children.size(); ++i)
		{
			if (((DebugObjectEditPart)children.get(i)).getModel().equals(model))
			{
				return (DebugObjectEditPart)children.get(i);	
			}	
		}
		return null;
	}

	protected void createEditPolicies()
	{
		installEditPolicy(EditPolicy.LAYOUT_ROLE, new EnclosingLayoutPolicy());
	}
	
	protected List getModelChildren()
	{
		return ((IDebugModel)getModel()).getObjects();
	}

	public void update(Observable model, Object msg)
	{
		m_refresher.refresh();
	}
}
