package cis690.insight.parts;

import java.util.List;
import java.util.Observable;
import java.util.Observer;

import org.eclipse.debug.core.model.IVariable;
import org.eclipse.draw2d.ConnectionAnchor;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.gef.editparts.AbstractGraphicalEditPart;

import cis690.insight.debug.IDebugObject;
import cis690.insight.figures.*;

/**
 * 
 * @version Apr 29, 2004
 * @author charlie
 */
public class DebugObjectEditPart extends AbstractGraphicalEditPart
	implements Observer, ObjFigListener
{
	private AsyncRefresh m_refresher;
	
	public DebugObjectEditPart(IDebugObject model)
	{
		setModel(model);
		m_refresher = new AsyncRefresh(this);
	}
	
	public void activate()
	{
		super.activate();
		((Observable)getModel()).addObserver(this);
	}
	
	public void deactivate()
	{
		super.deactivate();
		((Observable)getModel()).deleteObserver(this);
	}

	protected IFigure createFigure()
	{
		IDebugObject obj = (IDebugObject)getModel();
		ObjFig fig = new ObjFig(obj.getName(), obj.isThread());
		fig.setObjListener(this);
		return fig;
	}
	
	public ConnectionAnchor getSourceAnchor(IDebugObject model)
	{
		return ((ObjFig)getFigure()).getSourceAnchor(model.getObj());
	}
	
	public ConnectionAnchor getTargetAnchor()
	{
		return ((ObjFig)getFigure()).getAnchor();	
	}
	
	public void refreshVisuals()
	{
		super.refreshVisuals();
		
		IDebugObject model = (IDebugObject)getModel();
		ObjFig fig = (ObjFig)getFigure();
		
		//@TODO why do I have to make this check?  Am it not dying gracefully?
		if (getParent() == null)
		{
			return;	
		}
		
		// update the figure's constraints
		IFigure parent = fig.getParent();
		Rectangle bounds = (Rectangle)parent.getLayoutManager().getConstraint(fig);
		bounds.x = model.getXY().x;
		bounds.y = model.getXY().y;
		if (model.isShade())
		{
			bounds.height = fig.getHeaderHeight();
		}
		else
		{
			bounds.height = -1;	
		}
		parent.setConstraint(fig, bounds);
		
		// update the variables and children if we are suspended
		if (model.isSuspended())
		{
			fig.setVars(model.getVars());
		}
		//System.out.println(model.isSuspended() + "::refreshing::"+getModel().toString());
	}
	
	protected void createEditPolicies()
	{
		
	}

	// important that this not be named: "getModelSourceConnections()" or
	// "getModelChildren()" or "getChildren()" so we don't get auto behavior
	protected List getObjChildren()
	{
		return ((IDebugObject)getModel()).getChildren();
	}
	
	// important that this not be named: "getModelTargetConnections()" so we
	// don't get some automatic behavior
	protected List getObjParents()
	{
		return ((IDebugObject)getModel()).getParents();
	}

	public void update(Observable model, Object msg)
	{
		m_refresher.refresh();
	}

	public void toggleShade()
	{
		IDebugObject obj = (IDebugObject)getModel();
		obj.setShade(!obj.isShade());
		((Observable)obj).notifyObservers();
	}

	public void toggleVar(IVariable var)
	{
		IDebugObject model = (IDebugObject)getModel();
		if (model.getChildren().contains(var))
		{
			model.close(var);
		}
		else
		{
			model.open(var);
		}
		((Observable)model).notifyObservers();
	}

}
